// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:chopper/chopper.dart';
import 'package:matrix_sdk/src/api/client.dart';
import 'package:matrix_sdk/src/api/exception_interceptor.dart';
import 'package:matrix_sdk/src/exception.dart';

import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/user.dart';

/// Represents a Matrix homeserver. Also used as the main entry point
/// of the SDK.
class Homeserver {
  final ChopperClient _chopper;
  // TODO: Make lazy
  ClientService get _client => _chopper.getService();

  final Uri uri;

  Homeserver(this.uri) : _chopper = ChopperClient(
    baseUrl: uri.toString(),
    services: [
      ClientService.create()
    ],
    interceptors: [
      ExceptionInterceptor()
    ],
    converter: JsonConverter(),
  );

  /// Logs in to this homeserver and returns a [LocalUser], which can be
  /// used for further operations.
  ///
  /// [user]: Can be either a [UserId] or just a [Username].
  /// [password]: Password of the given [user].
  Future<LocalUser> login(UserIdentifier user, String password) async {

    final response = await _client.login({
      'type': 'm.login.password',
      'identifier': user.toIdentifierJson(),
      'password': password
    });

    return LocalUser(this, response.body['access_token']);
  }

  /// Checks whether the given [username] is available for
  /// registration on this [Homeserver].
  Future<bool> isUsernameAvailable(Username username) async {
    var response;

    try {
      response = await _client.registerAvailable(username.toString());
    } on UsernameInUseException {
      return false;
    }


    switch (response.statusCode) {
      case 200: return true;
      default: return false;
    }
  }
}